//+------------------------------------------------------------------+
//|                                 CandleWicksDisplayPercentage.mq4 |
//|                                  Copyright  2013, Andriy Moraru |
//+------------------------------------------------------------------+
#property copyright "Copyright  2013, Andriy Moraru"
#property link      "http://www.earnforex.com"

/*
   Alerts you when the candle's wick (shadow) reaches a certain length,
   expressed in percentage of the whole candle length.
   Your e-mail settings are set in Tools -> Options -> Email.
   Also displays the candle wicks' length above and below the candles.
*/

// The indicator uses only objects for display, but the line below is required for it to work.
#property indicator_chart_window

extern double DisplayWickLimit = 5; // In percentage points
extern color DisplayHighWickColor = Red;
extern color DisplayLowWickColor = LimeGreen;
extern int DisplayWickDistance = 10; // Distance from High to Pip Count in standard pips
extern double UpperWickAlertLimit = 50; // In percentage points
extern double LowerWickAlertLimit = 50; // In percentage points
extern bool WaitForClose = true; // Wait for a candle to close before checking wicks' length
extern bool EmailAlert = true;
extern bool SoundAlert = false;
extern bool VisualAlert = false;
extern int TopBottomPercent = 0; // if > 0 and <= 100, displays length only for bars where Open & Close within top or bottom % of candle

// Time of the bar of the last alert
datetime AlertDone;

double Poin;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
	Poin = Point;
	//Checking for unconvetional Point digits number
   if ((Point == 0.00001) || (Point == 0.001)) Poin *= 10;
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   for (int i = 0; i < Bars; i++)
   {
      ObjectDelete("Red-" + TimeToStr(Time[i]));
      ObjectDelete("Green-" + TimeToStr(Time[i]));
   }
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   string name, length;
   int index = 0;
   
   if (WaitForClose) index = 1;
   
   int counted_bars = IndicatorCounted();
   if (counted_bars > 0) counted_bars--;
   int limit = Bars - counted_bars;
   if (limit > 500) limit = 500;
     
   for (int i = 0; i <= limit; i++)
   {
      if ((TopBottomPercent > 0) && (TopBottomPercent <= 100))
      {
         double percent = (High[i] - Low[i]) * TopBottomPercent / 100;
         if (!(((Open[i] >= High[i] - percent) && (Close[i] >= High[i] - percent)) ||
             ((Open[i] <= Low[i] + percent) && (Close[i] <= Low[i] + percent)))) continue;
      }
      
      if (High[i] - Low[i] == 0) continue; // Skip bars with zero height
      
      double BodyHigh = MathMax(Open[i], Close[i]);
      double BodyLow = MathMin(Open[i], Close[i]);
      double UpperPercentage = ((High[i] - BodyHigh) / (High[i] - Low[i])) * 100;
      double LowerPercentage = ((BodyLow - Low[i]) / (High[i] - Low[i])) * 100;
      
      if (UpperPercentage >= DisplayWickLimit) // Upper wick length display
      {
         name = "Red-" + TimeToStr(Time[i]);
         length = DoubleToStr(MathRound(((High[i] - BodyHigh) / (High[i] - Low[i])) * 100), 0);
         if (ObjectFind(name) != -1) ObjectDelete(name);
         ObjectCreate(name, OBJ_TEXT, 0, Time[i], High[i] + DisplayWickDistance * Poin);
         ObjectSetText(name, length, 10, "Verdana", DisplayHighWickColor);
      }
      if (LowerPercentage >= DisplayWickLimit) // Lower wick length display
      {
         name = "Green-" + TimeToStr(Time[i]);
         length = DoubleToStr(MathRound(((BodyLow - Low[i]) / (High[i] - Low[i])) * 100), 0);
         if (ObjectFind(name) != -1) ObjectDelete(name);
         ObjectCreate(name, OBJ_TEXT, 0, Time[i], Low[i]);
         ObjectSetText(name, length, 10, "Verdana", DisplayLowWickColor);
      }
   }
   
   if (AlertDone == Time[index]) return(0); // Already sent an alert for this candle
   
   BodyHigh = MathMax(Open[index], Close[index]);
   BodyLow = MathMin(Open[index], Close[index]);
   UpperPercentage = ((High[index] - BodyHigh) / (High[index] - Low[index])) * 100;
   LowerPercentage = ((BodyLow - Low[index]) / (High[index] - Low[index])) * 100;
   
   if ((UpperPercentage >= UpperWickAlertLimit) || (LowerPercentage >= LowerWickAlertLimit))
   {
      datetime tc = TimeCurrent();
      string time = TimeToStr(tc);
      if (VisualAlert) Alert(time + " - wick limit reached!");
      if (SoundAlert) PlaySound("alert.wav");
      if (EmailAlert) SendMail("CandleWick Alert", time + " - wick limit reached!");
      AlertDone = Time[index];
   }
      
   return(0);
}
//+------------------------------------------------------------------+